<?php
require_once('class_lbCache.php');

class diskCache extends lbCache {
	
    /**
     * File based caching setting.
     */
    var $dir	= '/cache/lbc';	// Directory where lbcache must store 
									// generated files. Please use a dedicated
									// directory, and make it writable
    var $extension	= ".lbc";		// Extension used in the filename. This enables
									// us to (more accuratly) recognize lbcache-
									// files.
	
	function diskCache()
	{
		parent::lbCache();
	}
	
    /* restore()
     *
     * Will (try to) restore the cachedata.
     */
    function restore()
    {
		global $Config_root_dir;
        // Construct filename
        $filename = $Config_root_dir.$this->dir."/".$this->key.$this->extension;
		
        $contents = $this->fileread($filename);
        if ( is_null($contents) )
        {
            // Invalid cache-file
            $this->set_log("Invalid content of cache-file $filename");
			return FALSE;
        }
		// script modification time
		$LastChangeTime = filemtime($filename);
		if ( $LastChangeTime && (time() - $LastChangeTime) <= $this->expireTime )
		{
			// Set last modified (this helps search engines 
			// and other web tools to determine if a page has
			// been updated)
			@header('Last-Modified: '.gmdate('D, d M Y H:i:s', $LastChangeTime).' GMT');
			// Generate unique Hash-ID
			$myETag = substr($this->key,0,16).$LastChangeTime;
			// Send a "ETag" which represents the content
			// (this helps browsers to determine if the page
			// has been changed or if it can be loaded from
			// the cache - this will speed up the page loading)
			@header("ETag: \"$myETag\"");
			// First check if we can send last-modified
			// The browser "asks" us if the requested page has
			// been changed and sends the last modified date he
			// has in it's internal cache. So now we can check
			// if the submitted time equals our internal time value.
			// If yes then the page did not get updated
			$pageNotModified = ( isset($_SERVER["HTTP_IF_MODIFIED_SINCE"]) and strtotime($_SERVER["HTTP_IF_MODIFIED_SINCE"]) == $LastChangeTime );
			// The second possibility is that the browser sends us
			// the last Hash-ID he has. If he does we can determine
			// if he has the latest version by comparing both IDs. 
			$foundETag = ( isset($_SERVER["HTTP_IF_NONE_MATCH"]) and strstr($_SERVER["HTTP_IF_NONE_MATCH"], $myETag) );
			// Does one of the two ways apply?
			if ($pageNotModified or $foundETag)
			{
				// Okay, the browser already has the
				// latest version of our page in his
				// cache. So just tell him that
				// the page was not modified and DON'T
				// send the content -> this saves bandwith and
				// speeds up the loading for the visitor
				if(stristr($_SERVER["SERVER_SOFTWARE"], "microsoft"))
				{
					// IIS has already sent a HTTP/1.1 200 by this stage for
					// some strange reason
					@header("Status: 304 Not Modified");
				}
				else
				{
					@header("HTTP/1.0 304 Not Modified");
				}
				// That's all, now close the connection:
				@header('Connection: close');
				// The magical part: 
				// No content here ;-) 
				// Just the headers
			}
			// Okay, the browser does not have the
			// latest version or does not have any
			// version cached. So we have to send him
			// the full page.
			//Restore data
			return $contents;
		}
		$this->set_log("Data in cachefile $filename has expired");
        return FALSE;
    }
	
    /* write()
     *
     * Will (try to) write out the cachedata to the db
	 *
	 * Write an item to the cache.
	 *
	 * @param string The name of the cache
	 * @param mixed The data to write to the cache item
	 * @return boolean True on success, false on failure
	 */
    function put($contents)
    {
		global $Config_root_dir;
		// Construct filename
		$filename = $Config_root_dir.$this->dir."/".$this->key.$this->extension;
        // Create cachedata
		// Lock file, ignore warnings as we might be creating this file
        // And write the data
		$cache_file = @fopen($filename, "wb+");
		if(! $cache_file){
			$this->set_log("Failed to open for write of $filename");
			return false;
		}
		@flock($cache_file, LOCK_EX);
//		$cache_contents = "<?php\n\n/** Generated Cache - Do Not Alter\n * Cache Name: $name\n * Generated: ".gmdate("r")."\n*/\n\n";
/*		$cache_contents .= "\$$name = ".var_export($contents, true).";\n\n ?>";*/
		fwrite($cache_file, $contents, strlen($contents));
		// Release lock
		@flock($cache_file, LOCK_UN);
		fclose($cache_file);
        
		$this->set_log("Successfully wrote cachefile $name");
        return true;
//		$this->set_log("Unable to write cachefile $name");
        
    }
	
    /* This internal function reads in the cache-file */
    function fileread($filename)
    {
        // php.net suggested I should use rb to make it work under Windows
        $fp = @fopen($filename, "rb");
        if (!$fp) 
        {
            $this->set_log("Failed to open for read of $filename");
            return NULL;
        }
        
        // Get a shared lock
        @flock($fp, LOCK_SH);
        
        $buff="";
        // Be gentle, so read in 4k blocks
        while ( ($tmp = fread($fp, 4096)) ) 
        {
            $buff.=$tmp;
        }
        
        // Release lock
        @flock($fp, LOCK_UN);
        fclose($fp);
        // Return
        return $buff;
    }
	
    /* do_gc()
     *
     * Performs the actual garbagecollection
     */
    function do_gc($mode = '')
    {
		global $Config_root_dir;
		if ($mode === 'FLUSHSINGLE')
		{
			// Flush single page!
			$filename = $Config_root_dir.$this->dir."/".$this->key.$this->extension;
			$deleted = @unlink($filename);
			if ($deleted)
			{
				$this->set_log("Successfully unlinked $filename");
			}
			else
			{
				$this->set_log("Failed to unlink $filename");
			}
			return;
		}
		$dp = opendir($Config_root_dir.$this->dir);
		
		// Can we access directory ?
		if (!$dp) $this->set_log("Error opening ". $Config_root_dir.$this->dir ." for garbage-collection");
		
		$start_ex = strlen($this->extension)*-1;
		while (!(($de=readdir($dp)) === FALSE)) 
		{
			// To get around strange php-strpos, add additional char
			// Only read lbcache-files.
			if ( substr($de,$start_ex) != $this->extension ) continue;
			
			$filename = $Config_root_dir.$this->dir . "/" . $de;
	
			// Flush expirable pages (non-zero)
			if ($mode === 'FLUSH')
			{
				// Flush all pages!
				// Unlink file, we do not need to get a lock
				$deleted = @unlink($filename);
				if ($deleted)
				{
					$this->set_log("Successfully unlinked $filename");
				}
				else
				{
					$this->set_log("Failed to unlink $filename");
				}
			}
			else
			{
				// Flush expired pages
				$filetime = filemtime($filename);
				if ($filetime && (time() - $filetime) > $this->expireTime) 
				{
					// Unlink file, we do not need to get a lock
					$deleted = @unlink($filename);
					if ($deleted)
					{
						$this->set_log("Successfully unlinked $filename");
					}
					else
					{
						$this->set_log("Failed to unlink $filename");
					}
				} 
			} //end if $mode
		} //end while
    }
	
}
?>