<?php
class lbCache
{
    var $on	=	1;	// Turn caching on/off
    var $post	=	0;	// Should POST's be cached (default OFF)
    var $gc	=	1;	// Probability % of garbage collection
    var $gzip_level	=	1;	// GZIPcompressionlevel to use (1=low,9=high)
	var $expireTime = 900;	// Default number of seconds to cache a page MAX_AGE_FILES
    var $debug	=	0;   // Turn debugging on/off
	var $logs	=	array();	// The errors that occurred when handling data.
    var $flush	=	1;	// Flush page from cache when admin is loged in
    var $cleankeys	=	0;	// Set to 1 to avoid hashing storage-key:
							// you can easily see cachefile-origin.
	
	/**
	 * Initialize cache
	 * Build cache data.
	 *
	 */
	function lbCache()
	{
		global $Config_offline,$Config_lifetime;

        // Force cache off when POST occured when you don't want it cached
        if (!$this->post && (count($_POST) > 0)) $this->on = 0;
		else if ($Config_lifetime < 1) $this->on = 0;
		else if ($Config_offline == 1) $this->on = 0;
		
		$this->expireTime = $Config_lifetime * 60;

        // Handle type-specific additional code if required
        $this->do_start();
		
        if (!$this->on)
        {
            $this->set_log("Cache has been disabled!");
			
			// Check garbagecollection
//			$this->gc();
			
			// Go generate page and wait for cache_process()
//			$tmp = (isset($this->key)) ? $this->key : 'NULL';
//			$this->set_log("Cachedata not found for ".$tmp." key, generating now");
			ob_start(array (  & $this , 'cache_process' ));
			ob_implicit_flush(0);
			return;
        }
		// Set cache/proxy informations:
		@header('Cache-Control: max-age=' . $this->expireTime); // must-revalidate
		@header('Expires: '.gmdate('D, d M Y H:i:s', time()+$this->expireTime).' GMT');
		@header('Pragma: cache');
		// set GET and POST keys
        $this->set_key();
		
		// flush a single page
		if ( $this->flush > 0 && isset($_SESSION['admin_id']) ){
			
			$this->do_gc('FLUSHSINGLE');
			$this->set_log("Flushed cachedata for ".$this->key);
			
			// Go generate page and wait for cache_process()
			$tmp = (isset($this->key)) ? $this->key : 'NULL';
			$this->set_log("Cachedata not found for ".$tmp." key, generating now");
			ob_start(array (  & $this , 'cache_process' ));
			ob_implicit_flush(0);
			return;
		}

        // Can we read the cached data for this key ?
        $this->set_log("Looking for cachedata for ".$this->key);
        if ($contents = $this->restore())
        {
            $this->set_log("Cachedata for ".$this->key." found, data restored");
			
            // Handle type-specific additional code if required
            $this->do_end();
            // Cache is valid and restored: flush it!
			print $this->flush($contents);
            exit;
        }
        else
        {
            // No cache data (yet) or unable to read
            $this->set_log("No (valid) cachedata for ".$this->key);
			
			// Check garbagecollection
            //$this->gc();
			
			// Go generate page and wait for cache_process()
			$tmp = (isset($this->key)) ? $this->key : 'NULL';
			$this->set_log("Cachedata not found for ".$tmp." key, generating now");
			ob_start(array (  &$this , 'cache_process' ));
			ob_implicit_flush(0);
			return;
        }
	}

	/**
	 * Add an error to the error array. Take a wild guess...
	 *
	 * @param string The error name.
	 */
    function set_log($s)
    {
        if ($this->debug) array_push($this->logs,$s);
	}

	/**
	 * Returns the error(s) that occurred when handling data.
	 *
	 * @return string|array An array of errors.
	 */
	function get_logs()
	{
		return $this->logs;
	}
	
    /* gc()
     *
     * Checks if garbagecollection is needed.
     */
    function gc()
    {
        // Should we garbage collect ?
        if ($this->gc > 0)
        {
            mt_srand(time(NULL));
            $precision=100000;
            // Garbagecollection probability
            if (  ( (mt_rand()%$precision)/$precision ) <= ($this->gc/100)  )
            {
                $this->set_log("GarbageCollection hit!");
                $this->do_gc();
            }
        }
    }
	
    /* do_start()
     *
     * Additional code that is executed before real lbcache-code kicks in
     */
    function do_start()
    {  
        // Add additional code you might require
    }
	
    /* do_gc()
     *
     * Performs the actual garbagecollection
     */
    function do_gc($mode = '')
    {
		
    }
	
    /* do_end()
     *
     * Additional code that is executed after caching has been performed,
     * but just before output is returned. No new output can be added!
     */
    function do_end()
    {
        // Add additional code you might require
    }
	
	function cache_process($contents)
	{
		if( function_exists('ob_gzhandler') && !(ini_get('zlib.output_compression')) ){
			$contents = gzencode($contents, $this->gzip_level);
		}
		
        $this->set_log("Start cache process");
		@header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
        // If the connection was aborted, do not write the cache.
        // We don't know if the data we have is valid, as the user
        // has interupted the generation of the page.
        // Also check if lbcache is not disabled
        if ((!connection_aborted()) && $this->on && ($this->expireTime >= 0))
        {
            $this->set_log("Writing cached data to storage");
            // write the cache with the current data
            $this->put($contents);
			//return implode('<br \>',$this->logs);
        }
		
        // Handle type-specific additional code if required
        $this->do_end();

        // Return flushed data
        return $this->flush($contents);
		
	}
	
    /* flush()
     *
     * Responsible for final flushing everything.
	 * if new task is exist then we must uncompress data.
     * When browser support gzip, it is tried to send
     * the gzipped data. If that is also not possible, we sadly have to
     * uncompress
     */
    function flush($contents)
    {
		global $Config_live_site;
		$gz_allow = (function_exists('ob_gzhandler') && !(ini_get('zlib.output_compression')));
		
		$now = time();
		$task = @mysql_result(mysql_query('SELECT `tid` FROM `tasks` WHERE `status` = "enable" AND `nextrun` <= '.$now.' ORDER BY `nextrun` ASC LIMIT 1'),0);
		//force to uncompress contents
		if(! empty($task) ){
			if($gz_allow)
				return mygzdecode($contents).'<img src="'.$Config_live_site.'/task.php" border="0" width="1" height="1" alt="" />';
			else return $contents.'<img src="'.$Config_live_site.'/task.php" border="0" width="1" height="1" alt="" />';
		}
		
		$encoding = '';
		if (!headers_sent() && !connection_aborted() && $gz_allow ){
			if (strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'x-gzip') !== false) $encoding = "x-gzip";
			elseif (strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip') !== false) $encoding = "gzip";
		}
		if ($encoding != ''){
			//browser support gzip lib and we send encode contents
			@header("Content-Encoding: $encoding"); 
			return $contents;
		}else if($gz_allow){
			//contents is encode and browser not support gzip lib
			// we decode contents then send it
			return mygzdecode($contents);
		}else{
			//contents is not encode
			return $contents;
		}
    }
    /* set_key()
     *
     * Returns a hashvalue for the current. Maybe md5 is too heavy,
     * so you can implement your own hashing-function.
     */
    function set_key()
    {
		if ($this->cleankeys)
		{
			$key = eregi_replace("[^A-Z,0-9,=]", "_", $this->scriptkey());
			$key .= ".".eregi_replace("[^A-Z,0-9,=]", "_", $this->varkey());
			if (strlen($key) > 255)
			{
				// Too large, fallback to md5!
				$key = md5($this->scriptkey().$this->varkey());
			}
		}
		else
		{
			$key = md5($this->scriptkey().$this->varkey());
		}
		$this->set_log("Cachekey is set to $key");
		$this->key = $key;
    }
	
    /* varkey()
     *
     * Returns a serialized version of POST & GET vars
     * If you want to take cookies into account in the varkey too,
     * add them inhere.
     */
    function varkey()
    {
        $varkey = "";
        if ($this->post){
			$varkey = "POST(";
			foreach($_POST as $key=>$val)
				$varkey .= $key.'='.$val.'&';
			$varkey = rtrim($varkey,'&');
            $varkey .= ')';
        }

        // serialize get values minus the flush parameter
        $get = array_diff($_GET, array('flush' => $this->flush_key));
		$varkey .= "GET(";
		foreach($get as $key=>$val)
			$varkey .= $key.'='.$val.'&';
		$varkey = rtrim($varkey,'&');
        $varkey .= ')';
        $this->set_log("Cache varkey is set to $varkey");
        return $varkey;
    }

    /* scriptkey()
     *
     * Returns the script-identifier for the request
     */
    function scriptkey()
    {
		$name = $_SERVER["PHP_SELF"]?$_SERVER["PHP_SELF"]:$_SERVER["SCRIPT_NAME"];
        $this->set_log("Cache scriptkey is set to $name");
		
        return $name;
    }
	
    function restore(){}
	
	/**
	 * Write an item to the cache.
	 *
	 * @param string The name of the cache
	 * @param mixed The data to write to the cache item
	 * @return boolean True on success, false on failure
	 */
    function put($contents){}
	
}
?>