<?php
require_once('class_lbCache.php');

class mysqlCache extends lbCache {
	
    /**
     * DB based caching settings. 
     */
	var $db_table = "datacache"; // Table that holds the data
	var $sql_link = NULL;
	

	function mysqlCache()
	{
		parent::lbCache();
	}

    /* db_connect()
     *
     * Makes connection to the database
     */
    function db_connect()
    {
		global $Config_dbhost,$Config_dbuser,$Config_dbpass;
		$this->sql_link = @mysql_connect($Config_dbhost,$Config_dbuser,$Config_dbpass);
    }

    /* db_disconnect()
     *
     * Closes connection to the database
     */
    function db_disconnect()
    {
        mysql_close($this->sql_link);
    }

    /* db_query($query)
     *
     * Executes a given query
     */
    function db_query($query)
    {
		global $Config_dbname;
        $this->set_log("Executing SQL-query " . $query);
        $ret = @mysql_db_query($Config_dbname,$query,$this->sql_link);
        return $ret;
    }

    /* restore()
     *
     * Will try to restore the cachedata from the db.
     */
    function restore()
    {
        $res = $this->db_query("SELECT `contents`, `modificationTime` FROM ".$this->db_table.
                                " WHERE `key` = '".addslashes($this->key).
                                "' AND `modificationTime` >= ".(time() - $this->expireTime)
                               );
        if ($res && mysql_num_rows($res))
        {
            if ($row = mysql_fetch_array($res))
            {
				// script modification time
				$LastChangeTime = $row["modificationTime"];
				// Set last modified (this helps search engines 
				// and other web tools to determine if a page has
				// been updated)
				@header('Last-Modified: '.gmdate('D, d M Y H:i:s', $LastChangeTime).' GMT');
				// Generate unique Hash-ID
				$myETag = substr($this->key,0,16).$LastChangeTime;
				// Send a "ETag" which represents the content
				// (this helps browsers to determine if the page
				// has been changed or if it can be loaded from
				// the cache - this will speed up the page loading)
				@header("ETag: \"$myETag\"");
				// First check if we can send last-modified
				// The browser "asks" us if the requested page has
				// been changed and sends the last modified date he
				// has in it's internal cache. So now we can check
				// if the submitted time equals our internal time value.
				// If yes then the page did not get updated
				$pageNotModified = ( isset($_SERVER["HTTP_IF_MODIFIED_SINCE"]) and strtotime($_SERVER["HTTP_IF_MODIFIED_SINCE"]) == $LastChangeTime );
				// The second possibility is that the browser sends us
				// the last Hash-ID he has. If he does we can determine
				// if he has the latest version by comparing both IDs. 
				$foundETag = ( isset($_SERVER["HTTP_IF_NONE_MATCH"]) and strstr($_SERVER["HTTP_IF_NONE_MATCH"], $myETag) );
				// Does one of the two ways apply?
				if ($pageNotModified or $foundETag)
				{
					// Okay, the browser already has the
					// latest version of our page in his
					// cache. So just tell him that
					// the page was not modified and DON'T
					// send the content -> this saves bandwith and
					// speeds up the loading for the visitor
					if(stristr($_SERVER["SERVER_SOFTWARE"], "microsoft"))
					{
						// IIS has already sent a HTTP/1.1 200 by this stage for
						// some strange reason
						@header("Status: 304 Not Modified");
					}
					else
					{
						@header("HTTP/1.0 304 Not Modified");
					}
					// That's all, now close the connection:
					@header('Connection: close');
					// The magical part: 
					// No content here ;-) 
					// Just the headers
				}
				// Okay, the browser does not have the
				// latest version or does not have any
				// version cached. So we have to send him
				// the full page.
				//Restore data
				$this->set_log("Restoring cachedata");
				
				return $row["contents"];
			}
        }
		$this->set_log("Data in mysql has expired");
        return false;
    }

    /* write()
     *
     * Will (try to) write out the cachedata to the db
     */
    function put($contents)
    {

        // XXX: Later on, maybe implement locking mechanism inhere.

        // Check if it already exists
        $res = $this->db_query("SELECT `modificationTime` FROM ".$this->db_table.
                                " WHERE `key` = '".addslashes($this->key)."'"
                               );


        if (!$res || mysql_num_rows($res) < 1)
        {
            // Key not found, so insert
            $this->set_log("Cache key not found, inserting now");
            $res = $this->db_query("INSERT INTO ".$this->db_table.
                                    " (`key`, `modificationTime`, `contents`) VALUES ('".
                                        addslashes($this->key)."',".time().",'".addslashes($contents)."')"
                                   );
            // This fails with unique-key violation when another thread has just
            // inserted the same key. Just continue, as the result is (almost)
            // the same.
        }
        else
        {
            // Key found, so update
            $this->set_log("Cache key found, updating now");
            $res = $this->db_query("UPDATE ".$this->db_table.
									" SET `modificationTime` = ".time().", `contents` = '".addslashes($contents).
									"' WHERE `key` = '".addslashes($this->key)."'"
                                   );
            // This might be an update too much, but it shouldn't matter
        }
    }

    /* do_gc()
     *
     * Performs the actual garbagecollection
     */
    function do_gc($mode = '')
    {

         // Flush expirable pages (non-zero)
         if ($mode === 'FLUSH')
         {
         	// Flush all pages!
            $this->db_query("DELETE FROM ".$this->db_table);
         }
         elseif ($mode === 'FLUSHSINGLE')
         {
         	// Flush single page!
            $this->db_query("DELETE FROM ".$this->db_table.
                           " WHERE `key` = '" . addslashes($this->key) . "'"
                            );
         }
         else
         {
         	// Flush expired pages
            $this->db_query("DELETE FROM ".$this->db_table.
                             " WHERE `modificationTime` < ".(time() - $this->expireTime)
                            );
         }

         $this->db_query("OPTIMIZE TABLE ".$this->db_table);
    }


    /* do_start()
     *
     * Additional code that is executed before main lbcache-code kicks in.
     */
    function do_start()
    {
        // Connect to db
        $this->db_connect();
    }

    /* do_end()
     *
     * Additional code that is executed after caching has been performed,
     * but just before output is returned. No new output can be added!
     */
    function do_end()
    {
        // Disconnect from db
        $this->db_disconnect();
    }
	
}
?>